#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <string.h>
#include <ctype.h>
#include <errno.h>
#include <signal.h>
#include <time.h>
#include <sys/kernel.h>
#include <sys/osinfo.h>
#include <Ph.h>

#ifdef __USAGE
%C - Photon Event Printer (QNX)

This program prints event lists created by by precord.

%C [options] <inputfile

Options:
    -n nid/name		Specify a Photon server

Example:
%C -n2 <abdemo.pev

#endif

#define EVENT_BUFFER_SIZE 64000
#define MILLITIME( time_sel )			( time_sel->seconds * 1000 + time_sel->nsec / 1000000 )


char 			PhotonDev[30] 			= "/dev/photon";
unsigned		Verbosity				= 0;
unsigned		Go						= 1;
int				ExitValue				= EXIT_SUCCESS;
PhRid_t			Rid, ptr_Rid;
char			*Buffer;
PhEvent_t		*Event;
struct _timesel far *TimePtr;

static init_system( void )
{
	PhRegion_t region, ptr_region;
	PhRect_t rect;
	PhArea_t area;
	struct _osinfo osdata;

	if( -1 == qnx_osinfo( 0, &osdata ) ) {
		fprintf( stderr, "osinfo failed\n" );
		exit( EXIT_FAILURE );
	}
	TimePtr = MK_FP( osdata.timesel, 0 );

	if( !PhAttach( PhotonDev, NULL ) ) {
		fprintf( stderr, "Could not attach to a Photon\n" );
		exit( EXIT_FAILURE );
	}

	if( NULL == ( Buffer = malloc( EVENT_BUFFER_SIZE ) ) ) {
		fprintf( stderr, "Could not allocate event buffer\n" );
		exit( EXIT_FAILURE );
	}

	return( 0 );
}


void Init( int argc, char *argv[] )
{
	int c, error = 0;

	while( ( c = getopt( argc, argv, "n:v" ) ) != -1 ) {
		switch( c ) {
			case 'n':
				if( isdigit( *optarg ) ) 
					sprintf( PhotonDev, "//%d/dev/photon", strtol( optarg, NULL, 0 ) );
				else {
					if( *optarg == '/' ) 
						strcpy( PhotonDev, optarg );
					else
						sprintf( PhotonDev, "/dev/%s", optarg );
				}
				break;
			case 'v':
				++Verbosity;
				break;
			default:
				error = 1;
		}
	}
	if( error ) {
		Go = 0;
		fprintf( stderr, "Error parsing options\n" );				
		return;
	}
		
	init_system();
}


void main( int argc, char *argv[] ) 
{
	unsigned used, reload, ev_len;
	long lasttime, lasttimestamp = 0, diff;
	unsigned long offset;
	struct timespec sleeptime;
    unsigned ecount = 0;
    PhRawPtrEvent_t *ptrdata;    /* raw pointer data */
    PhRawKeyEvent_t *keydata;    /* raw keyboard data */

	Init( argc, argv );

	offset = 0;

	while( Go ) {
		used = 0;
		Event = (PhEvent_t *)Buffer;
		fseek( stdin, offset, SEEK_SET );
		if( !fread( Buffer, 1, EVENT_BUFFER_SIZE, stdin ) ) {
			Go = 0;
			continue;
		}
		reload = 0;

		while( !reload ) {
			Event = (PhEvent_t *)( Buffer + used );

			if( EVENT_BUFFER_SIZE - used < sizeof( PhEvent_t ) ) {
				reload = 1;
				continue;
			}

			ev_len = sizeof( PhEvent_t ) +
						sizeof( PhRect_t ) * Event->num_rects +
						Event->data_len;

			if( EVENT_BUFFER_SIZE - used < ev_len ) {
				reload = 1;
				continue;
			}

			diff =  ( Event->timestamp - lasttimestamp ) - ( MILLITIME( TimePtr ) - lasttime );
			if( diff > 0 && lasttimestamp ) {
				sleeptime.tv_sec = diff / 1000;
				sleeptime.tv_nsec = ( diff % 1000 ) * 1000000;
                //printf("s:%ld %ld\n", sleeptime.tv_sec, sleeptime.tv_nsec); fflush(stdout);
				nanosleep( &sleeptime, NULL );
			}

			lasttime = MILLITIME( TimePtr );
			lasttimestamp = Event->timestamp;

            print_event_type(Event->type);
            print_event_subtype(Event->subtype);
            fprintf(stdout,"num_rects=%u, data_len=%u ev_len=%d",
                    Event->num_rects, Event->data_len, ev_len );

            switch ( Event->type ) {
                case Ph_EV_DRAW:
                              /*
    		                     Event->emitter.rid = Ph_DEV_RID;
                                 Event->flags = Ph_EMIT_TOWARD;
			                     Event->input_group = 1;
			                     Event->collector.rid = 0;
			                     PhEventEmit( Event, PhGetRects( Event ), PhGetData( Event ) );
                               */
                                 break;
                case Ph_EV_KEY:
                case Ph_EV_BUT_PRESS:
                case Ph_EV_BUT_RELEASE:
                case Ph_EV_PTR_MOTION_NOBUTTON:
                case Ph_EV_PTR_MOTION_BUTTON:
                                 break;
                case Ph_EV_RAW:
                               /* raw pointer event */
                               switch ( Event->subtype ) {
                                    case Ph_EV_RAW_KEY:
                                                        fprintf(stdout,"Key: ");
                                                        keydata = PhGetData( Event );
                                                        fprintf(stdout,"key: cap=%lu, sym=%lu flags=%lu mod=%lu ", 
                                                             keydata->key_cap,
                                                             keydata->key_sym,
                                                             keydata->key_flags,
                                                             keydata->key_mods
                                                             );
                                                        break;
                                    case Ph_EV_RAW_PTR:
                                                        fprintf(stdout,"Ptr: ");
                                                        ptrdata = PhGetData( Event );
                                                        fprintf(stdout,"button_state=%u flags=%x, raw_flags=%x ",
                                                             ptrdata->button_state, ptrdata->flags, ptrdata->raw_flags);
                                                        break;
                                    default:
                                                        break;
                                    }

                               /*
    		                     Event->emitter.rid = ptr_Rid;
			                     Event->input_group = 0xff;
			                     Event->collector.rid = 0;
			                     PhEventEmit( Event, PhGetRects( Event ), PhGetData( Event ) );
                                */
                                 break;
                case Ph_EV_SYSTEM:
                                 break;
                case Ph_EV_BOUNDARY:
                                 break;
                case Ph_EV_EXPOSE:
                                 break;
                default:
                                 printf("type=%lx\n", Event->type );
                                 break;
                }

			used += ev_len;
			offset += ev_len;
            printf("\n"); fflush(stdout);
		}
	}
	exit( ExitValue );
}

print_event_type( long event )
{
if ( event & Ph_EV_RAW )
   printf("RAW ");

if ( event & Ph_EV_BUT_PRESS )
   printf("BUT_PRESS ");

if ( event & Ph_EV_BUT_RELEASE )
   printf("BUT_RELEASE ");

if ( event & Ph_EV_PTR_MOTION_NOBUTTON )
   printf("PTR_MOTION_NOBUTTON ");

if ( event & Ph_EV_PTR_MOTION_BUTTON )
   printf("PTR_MOTION_BUTTON ");

if ( event & Ph_EV_BOUNDARY )
   printf("BOUNDARY ");

if ( event & Ph_EV_EXPOSE )
   printf("EXPOSE ");

if ( event & Ph_EV_DRAW )
   printf("DRAW ");

if ( event & Ph_EV_AMIN )
   printf("AMIN ");

if ( event & Ph_EV_DRAG )
   printf("DRAG ");

if ( event & Ph_EV_COVERED )
   printf("COVERED ");

if ( event & Ph_EV_BLIT )
   printf("BLIT ");

if ( event & Ph_EV_SYSTEM )
   printf("SYSTEM ");

if ( event & Ph_EV_WM )
   printf("WM ");

if ( event & Ph_EV_BUT_REPEAT )
   printf("BUT_REPEAT ");

if ( event & Ph_EV_RAW )
   printf("RAW ");
}

print_event_subtype( unsigned short event )
{
printf("Subtype %u ", event);
}

