#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <string.h>
#include <ctype.h>
#include <errno.h>
#include <signal.h>
#include <time.h>
#include <sys/kernel.h>
#include <sys/osinfo.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <Ph.h>

#ifdef __USAGE
%C - Photon Player (QNX)

This program plays back draw event files generated by
precord.

%C [options] <inputfile

Options:
    -n nid/name		Specify a Photon server
    -i              Allow input events to go through.. Default is to NOT pass
                    button clicks or key events through.
    -d              Debug to stdout

Example:
%C -n2 <abdemo.pev

#endif

#define EVENT_BUFFER_SIZE 64000
#define MILLITIME( time_sel )	( time_sel->seconds * 1000 + time_sel->nsec / 1000000 )


char 			PhotonDev[30] 			= "/dev/photon";
unsigned		Verbosity				= 0;
unsigned		Go						= 1;
int				ExitValue				= EXIT_SUCCESS;
PhRid_t			Rid, ptr_Rid;
char			*Buffer;
PhEvent_t		*Event;
struct _timesel far *TimePtr;
char            inputpass               = 0; /* default is not to pass input events */         
char            debug                   = 0;       
FILE            *fpin;

static init_system( void )
{
	PhRegion_t region, ptr_region;
	PhRect_t rect;
	PhArea_t area;
	struct _osinfo osdata;

	if( -1 == qnx_osinfo( 0, &osdata ) ) {
		fprintf( stderr, "osinfo failed\n" );
		exit( EXIT_FAILURE );
	}
	TimePtr = MK_FP( osdata.timesel, 0 );

	if( !PhAttach( PhotonDev, NULL ) ) {
		fprintf( stderr, "Could not attach to a Photon\n" );
		exit( EXIT_FAILURE );
	}

    fpin = stdin;
    if ( fpin == NULL ) {
       fprintf(stderr,"Can't open stdin\n");
       exit(1);
       }

/* draw */
	memset( &region, 0, sizeof( region ) );
	region.parent = Ph_DEV_RID;
	region.flags = Ph_PTR_REGION;
	region.input_group = 1;
	rect.ul.x = rect.ul.y = SHRT_MIN;
	rect.lr.x = rect.lr.y = SHRT_MAX;
	if( !( Rid = PhRegionOpen( 
		Ph_REGION_PARENT |
		Ph_REGION_FLAGS |
		Ph_REGION_RECT, &region, &rect, NULL ) ) ) {
		fprintf( stderr, "Could not open region\n" );
		exit( EXIT_FAILURE );
	}

/* pointer */
	memset( &ptr_region, 0, sizeof( ptr_region ) );
	ptr_region.flags = Ph_PTR_REGION;
	ptr_region.input_group = 1;
	rect.ul.x = rect.ul.y = rect.lr.x = rect.lr.y = 0;
	if( !( ptr_Rid = PhRegionOpen( 
		Ph_REGION_FLAGS |
		Ph_REGION_INPUT_GROUP |
		Ph_REGION_RECT, &ptr_region, &rect, NULL ) ) ) {
		fprintf( stderr, "Could not open region\n" );
		exit( EXIT_FAILURE );
	}

	if( NULL == ( Buffer = malloc( EVENT_BUFFER_SIZE ) ) ) {
		fprintf( stderr, "Could not allocate event buffer\n" );
		exit( EXIT_FAILURE );
	}

	return( 0 );
}


void Init( int argc, char *argv[] )
{
	int c, error = 0;

	while( ( c = getopt( argc, argv, "n:vid" ) ) != -1 ) {
		switch( c ) {
			case 'n':
				if( isdigit( *optarg ) ) 
					sprintf( PhotonDev, "//%d/dev/photon", 
                             strtol( optarg, NULL, 0 ) );
				else {
					if( *optarg == '/' ) 
						strcpy( PhotonDev, optarg );
					else
						sprintf( PhotonDev, "/dev/%s", optarg );
				}
				break;
			case 'v':
				++Verbosity;
				break;
            case 'i':
                inputpass = 1;
                break;
            case 'd':
                debug = 1;
                break;
			default:
				error = 1;
		}
	}
	if( error ) {
		Go = 0;
		fprintf( stderr, "Error parsing options\n" );				
		return;
	}
		
	init_system();
}


void main( int argc, char *argv[] ) 
{
	unsigned used, reload, ev_len;
	long lasttime, lasttimestamp = 0, diff;
	unsigned long offset;
	struct timespec sleeptime;
    unsigned ecount = 0;
    PhRawPtrEvent_t *ptrdata;    /* raw pointer data */
    PhRawKeyEvent_t *keydata;    /* raw keyboard data */
    unsigned short last_button_state = 0;   
                                 /* record state of last button press */

	Init( argc, argv );

	offset = 0;

	while( Go ) {
		used = 0;
		Event = (PhEvent_t *)Buffer;
		fseek( fpin, offset, SEEK_SET );
		if( !fread( Buffer, 1, EVENT_BUFFER_SIZE, fpin ) ) {
			Go = 0;
			continue;
		}
		reload = 0;

		while( !reload ) {
			Event = (PhEvent_t *)( Buffer + used );

			if( EVENT_BUFFER_SIZE - used < sizeof( PhEvent_t ) ) {
				reload = 1;
				continue;
			}

			ev_len = sizeof( PhEvent_t ) +
						sizeof( PhRect_t ) * Event->num_rects +
						Event->data_len;

			if( EVENT_BUFFER_SIZE - used < ev_len ) {
				reload = 1;
				continue;
			}

			diff =  ( Event->timestamp - lasttimestamp ) - ( MILLITIME( TimePtr ) - lasttime );
			if( diff > 0 && lasttimestamp ) {
				sleeptime.tv_sec = diff / 1000;
				sleeptime.tv_nsec = ( diff % 1000 ) * 1000000;
                //printf("s:%ld %ld\n", sleeptime.tv_sec, sleeptime.tv_nsec); fflush(stdout);
				nanosleep( &sleeptime, NULL );
			}

			lasttime = MILLITIME( TimePtr );
			lasttimestamp = Event->timestamp;

            if ( debug ) {
               printevent();
               }

            switch ( Event->type ) {
                case Ph_EV_DRAW:
    		                     Event->emitter.rid = Ph_DEV_RID;
                                 Event->flags = Ph_EMIT_TOWARD;
			                     Event->input_group = 1;
			                     Event->collector.rid = 0;
			                     PhEventEmit( Event, PhGetRects( Event ), PhGetData( Event ) );
                                 break;
                case Ph_EV_KEY:
                case Ph_EV_BUT_PRESS:
                case Ph_EV_BUT_RELEASE:
                                 if ( inputpass ) {
    		                        Event->emitter.rid = ptr_Rid;
			                        Event->input_group = 0xff;
			                        Event->collector.rid = 0;
			                        PhEventEmit( Event, PhGetRects( Event ), PhGetData( Event ) );
                                    }
                                 break;
                case Ph_EV_PTR_MOTION_BUTTON:
                case Ph_EV_PTR_MOTION_NOBUTTON:
    		                     Event->emitter.rid = ptr_Rid;
			                     Event->input_group = 0xff;
			                     Event->collector.rid = 0;
			                     PhEventEmit( Event, PhGetRects( Event ), PhGetData( Event ) );
                                 break;
                case Ph_EV_RAW:
                               /* raw pointer event */
                               switch ( Event->subtype ) {
                                    case Ph_EV_RAW_KEY:
                                                        keydata = PhGetData( Event );
                                                        //fprintf(stdout,"key=%lu ", 
                                                        //     keydata->key_cap);
                                                        break;
                                    case Ph_EV_RAW_PTR:
                                                        ptrdata = PhGetData( Event );
                                                        if ( ptrdata->button_state == 0 || ptrdata->button_state == last_button_state ) {
    		                                               Event->emitter.rid = ptr_Rid;
			                                               Event->input_group = 0xff;
			                                               Event->collector.rid = 0;
			                                               PhEventEmit( Event, PhGetRects( Event ), PhGetData( Event ) );
                                                           }
                                                        else 
                                                        if ( inputpass ) {
    		                                               Event->emitter.rid = ptr_Rid;
			                                               Event->input_group = 0xff;
			                                               Event->collector.rid = 0;
			                                               PhEventEmit( Event, PhGetRects( Event ), PhGetData( Event ) );
                                                           }
                                                        last_button_state = ptrdata->button_state;
                                                        break;
                                    default:
                                                        break;
                                    }
                                 break;
                case Ph_EV_SYSTEM:
                                 break;
                case Ph_EV_BOUNDARY:
                                 break;
                case Ph_EV_EXPOSE:
                                 break;
                default:
                                 //printf("type=%lx\n", Event->type );
                                 break;
                }

			used += ev_len;
			offset += ev_len;
		}
	}
	exit( ExitValue );
}

printevent()     /* display Event information to stdout */
{
PhRawPtrEvent_t *ptrdata;    /* raw pointer data */
PhRawKeyEvent_t *keydata;    /* raw keyboard data */

print_event_type( Event->type );
printf("Subtype %u ", Event->subtype);

switch ( Event->type ) {
   case Ph_EV_KEY:
   case Ph_EV_BUT_PRESS:
   case Ph_EV_BUT_RELEASE:
                    break;
   case Ph_EV_PTR_MOTION_BUTTON:
   case Ph_EV_PTR_MOTION_NOBUTTON:
                    break;
   case Ph_EV_RAW:
                  /* raw pointer event */
                  switch ( Event->subtype ) {
                       case Ph_EV_RAW_KEY:
                                           keydata = PhGetData( Event );
                                           printf("key=%lu ", 
                                                keydata->key_cap);
                                           break;
                       case Ph_EV_RAW_PTR:
                                           ptrdata = PhGetData( Event );
                                           printf("button_state=%u ", 
                                                ptrdata->button_state);
                                           break;
                       default:
                                           break;
                       }
                    break;
   case Ph_EV_SYSTEM:
                    break;
   case Ph_EV_BOUNDARY:
                    break;
   case Ph_EV_EXPOSE:
                    break;
   default:
                    printf(" type=%lx", Event->type );
                    break;
   }
   printf("\n");
   fflush(stdout);
   getchar();
}



print_event_type( long event )
{
if ( event & Ph_EV_RAW )
   printf("RAW ");

if ( event & Ph_EV_BUT_PRESS )
   printf("BUT_PRESS ");

if ( event & Ph_EV_BUT_RELEASE )
   printf("BUT_RELEASE ");

if ( event & Ph_EV_PTR_MOTION_NOBUTTON )
   printf("PTR_MOTION_NOBUTTON ");

if ( event & Ph_EV_PTR_MOTION_BUTTON )
   printf("PTR_MOTION_BUTTON ");

if ( event & Ph_EV_BOUNDARY )
   printf("BOUNDARY ");

if ( event & Ph_EV_EXPOSE )
   printf("EXPOSE ");

if ( event & Ph_EV_DRAW )
   printf("DRAW ");

if ( event & Ph_EV_AMIN )
   printf("AMIN ");

if ( event & Ph_EV_DRAG )
   printf("DRAG ");

if ( event & Ph_EV_COVERED )
   printf("COVERED ");

if ( event & Ph_EV_BLIT )
   printf("BLIT ");

if ( event & Ph_EV_SYSTEM )
   printf("SYSTEM ");

if ( event & Ph_EV_WM )
   printf("WM ");

if ( event & Ph_EV_BUT_REPEAT )
   printf("BUT_REPEAT ");
}
