#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <string.h>
#include <ctype.h>
#include <errno.h>
#include <signal.h>
#include <sys/kernel.h>
#include <Ph.h>

#ifdef __USAGE
%C - Photon Recorder (QNX)

This program records all the Photon draw events that 
pass through its region.  pplay can be used to play
the resulting event log back.

%C [options] >outputfile
Options:
    -n nid/name		Specify node to record

Example:
%C -n2 >abdemo.pev

#endif

#define EVENT_BUFFER_SIZE 64000

char 			PhotonDev[30] 			= "/dev/photon";
unsigned		Verbosity				= 0;
unsigned		Go						= 1;
int				ExitValue				= EXIT_SUCCESS;
PhRid_t			Rid, ptr_Rid, irid;
PhEvent_t		*Event;
char			*VBuf;

PhPoint_t		Origin					= { 0, 0 };
unsigned char	InputGroup				= 0xff;

void SignalExit( int signo )
{
	fflush( NULL );
	Go = 0;
}

static init_system( void )
{
	PhRegion_t region, ptr_region;
	PhRect_t rect;
	PhRect_t area;
    unsigned flags;

	if( !PhAttach( PhotonDev, NULL ) ) {
		fprintf( stderr, "Could not attach to a Photon\n" );
		exit( EXIT_FAILURE );
	}

/*
	PhQueryRids(0, 0, ((grp = getenv("PHIG")) != NULL) ? atoi(grp) : 1, 
                Ph_INPUTGROUP_REGION | Ph_QUERY_EXACT, 0, 0, 0, &irid, 1);
	PhRegionQuery(irid, NULL, &rect, NULL, 0);
*/

	if( VBuf = malloc( 12288 ) )
		setvbuf( stdout, VBuf, _IOFBF, 12288 );

    /* region is for draw events
       ptr_region is for pointer/keyboard events 
     */

	flags = Ph_REGION_EV_OPAQUE | Ph_REGION_EV_SENSE | Ph_REGION_FLAGS;
	memset( &region, 0, sizeof region );
	memset( &area, 0, sizeof area );
	region.flags = Ph_FORCE_FRONT | Ph_GRAFX_REGION;
	region.events_sense = Ph_EV_DRAW | Ph_EV_DRAG;
	region.events_opaque = 0;

	region.input_group = -1;
	flags |= Ph_REGION_INPUT_GROUP;

	region.origin.x = 0;
	region.origin.y = 0;
	flags |= Ph_REGION_ORIGIN;

	area.ul.x = 0;
	area.ul.y = 0;
	area.lr.x = 1023;
	area.lr.y = 767;
	flags |= Ph_REGION_RECT;

	Rid = PhRegionOpen( flags, &region, &area, NULL );

    if ( Rid == -1 ) {
		fprintf( stderr, "Could not open region\n" );
		exit( EXIT_FAILURE );
	}

/* pointer/keyboard events */
	memset( &ptr_region, 0, sizeof( ptr_region ) );
	if( 1 != PhQueryRids( 0, 0, 1, Ph_INPUTGROUP_REGION, 0, 0, 0, 
           &ptr_region.parent, 1 ) ) {
		fprintf( stderr, "PhQueryRids error.\n" );
		exit( EXIT_FAILURE );
	}

	ptr_region.events_sense = Ph_EV_RAW;

	rect.ul.x = rect.ul.y = 0;
	rect.lr.x = 1023;
	rect.lr.y = 767;
	if( !( ptr_Rid = PhRegionOpen( 
		Ph_REGION_PARENT |
		Ph_REGION_EV_SENSE |
		Ph_REGION_BEHIND |
		Ph_REGION_RECT, &ptr_region, &rect, NULL ) ) ) {
		fprintf( stderr, "Could not open region\n" );
		exit( EXIT_FAILURE );
	}

	if( NULL == ( Event = malloc( EVENT_BUFFER_SIZE ) ) ) {
		fprintf( stderr, "Could not allocate event buffer\n" );
		exit( EXIT_FAILURE );
	}

    /* emit expose event to the whole system to get the first set of
       draws 
     */
     EmitExpose( Rid );

	signal( SIGINT, SignalExit );

	return( 0 );
}

void EmitExpose( PhRid_t rid )
{
	PhEvent_t			event;
	PhRect_t			rect;
	
	memset( &event, 0, sizeof( event ) );
	event.type = Ph_EV_EXPOSE;
	event.emitter.rid = rid;
	event.num_rects = 1;
	rect.ul.x = rect.ul.y = SHRT_MIN;
	rect.lr.x = rect.lr.y = SHRT_MAX;
	PhEventEmit( &event, &rect, NULL );
}

void Init( int argc, char *argv[] )
{
	int c, error = 0;

	while( ( c = getopt( argc, argv, "n:v" ) ) != -1 ) {
		switch( c ) {
			case 'n':
				if( isdigit( *optarg ) ) 
					sprintf( PhotonDev, "//%d/dev/photon", 
                             strtol( optarg, NULL, 0 ) );
				else {
					if( *optarg == '/' ) 
						strcpy( PhotonDev, optarg );
					else
						sprintf( PhotonDev, "/dev/%s", optarg );
				}
				break;
			case 'v':
				++Verbosity;
				break;
			default:
				error = 1;
		}
	}
	if( error ) {
		Go = 0;
		fprintf( stderr, "Error parsing options\n" );				
		return;
	}
		
	init_system();
}


void main( int argc, char *argv[] ) 
{
	Init( argc, argv );

	while( Go ) {
		switch( PhEventNext( Event, EVENT_BUFFER_SIZE ) ) {
			case Ph_EVENT_MSG:
                /* now we got an event */
/*
                fprintf(stdout,"type=%lx, %x, time=%lx, numr=%u, dlen=%u\n",
                      Event->type, Event->subtype, Event->timestamp,
                      Event->num_rects, Event->data_len);
*/

				fwrite( Event, 1, sizeof( PhEvent_t ) + 
									Event->num_rects * sizeof( PhRect_t ) +
									Event->data_len, stdout );
				break;
		}
	}
	exit( ExitValue );
}

